<?php

declare(strict_types=1);

namespace Drupal\message_recipient_group\Plugin\MessageRecipient\RecipientsCollector;

use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\group\Entity\GroupRelationshipInterface;
use Drupal\message\MessageInterface;
use Drupal\message_recipient\Attribute\RecipientsCollector;
use Drupal\message_recipient_entity\ContentEntityUserRecipientsCollectorBase;
use Drupal\user\EntityOwnerInterface;

/**
 * Group owner recipients collector.
 */
#[RecipientsCollector(
  id: 'group_owner',
  type: 'user',
  category: 'group',
  allow_multiple: FALSE,
  label: new TranslatableMarkup('Group owner by group relationship'),
  description: new TranslatableMarkup('Collects group owner from the message\'s associated "group relationship" entity.'),
)]
class GroupOwner extends ContentEntityUserRecipientsCollectorBase {

  /**
   * {@inheritdoc}
   */
  public function collect(MessageInterface $message): array {
    return array_values($this->query($message)->execute());
  }

  /**
   * {@inheritdoc}
   */
  public function query(MessageInterface $message): QueryInterface {
    $query = parent::query($message);
    $query->condition('uid', $this->getAssociatedEntity($message)->getGroup()->getOwnerId());
    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function isApplicable(MessageInterface $message): bool {
    if ($this->hasAssociatedEntityField($message)) {
      $associated_entity = $this->getAssociatedEntity($message);
      return $associated_entity instanceof GroupRelationshipInterface
      && $associated_entity->getGroup() instanceof EntityOwnerInterface;
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary(): array {
    return [
      'recipient_collector' => [
        '#markup' => $this->description(),
      ],
    ] + parent::getSummary();
  }

}
