<?php

declare(strict_types=1);

namespace Drupal\message_recipient_group\Plugin\MessageRecipient\RecipientsCollector;

use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\group\Entity\GroupRelationshipInterface;
use Drupal\message\MessageInterface;
use Drupal\message_recipient\Attribute\RecipientsCollector;
use Drupal\message_recipient_entity\ContentEntityUserRecipientsCollectorBase;

/**
 * User by group user relationship recipients' collector.
 */
#[RecipientsCollector(
  id: 'user_by_group_user_relationship',
  type: 'user',
  category: 'group',
  label: new TranslatableMarkup('User by group user relationship'),
  description: new TranslatableMarkup("Collects user by group user relationship from the message's associated."),
)]
class UserByGroupUserRelationship extends ContentEntityUserRecipientsCollectorBase {

  /**
   * {@inheritdoc}
   */
  public function collect(MessageInterface $message): array {
    return array_values($this->query($message)->execute());
  }

  /**
   * {@inheritdoc}
   */
  public function query(MessageInterface $message): QueryInterface {
    $query = parent::query($message);
    $query->condition('uid', $this->getAssociatedEntity($message)->getEntity()->id());
    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function isApplicable(MessageInterface $message): bool {
    if ($this->hasAssociatedEntityField($message)) {
      $associated_entity = $this->getAssociatedEntity($message);
      return $associated_entity instanceof GroupRelationshipInterface
        && $associated_entity->getPlugin()->getRelationType()->getEntityTypeId() === 'user';
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary(): array {
    return [
      'recipient_collector' => [
        '#markup' => $this->description(),
      ],
    ] + parent::getSummary();
  }

}
